﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Interop;
using System.Collections.Generic;

namespace InkPresentationSample_UserControl
{
    public partial class Page : Canvas
    {
        // 프로퍼티

        // 배경 이미지
        int _MaxBackImage = 0;
        List<Uri> _listImageUri = new List<Uri>();

        // Ink 작품 리스트
        List<UCInk> _listInkWork = new List<UCInk>();

        // Ink Presenter 편집기
        Size _szInk = new Size(300, 200);
        UCInk _oInkEditor = null;

        // 랜덤 값 발생기
        Random _oRandom = new Random();

        public void Page_Loaded(object o, EventArgs e)
        {
            // Required to initialize variables
            InitializeComponent();

            BrowserHost.Resize += new EventHandler(BrowserHost_Resize);

            // 배경이미지 준비
            LoadBackImages();

            // Editor 생성
            CreateInkEditor();

            // 이벤트 연결
            ConnectEditorEvents();
        }

        void BrowserHost_Resize(object sender, EventArgs e)
        {
            Width = BrowserHost.ActualWidth;
            Height = BrowserHost.ActualHeight;
        }

        void LoadBackImages()
        {
            for (int i = 1; i <= 5; i++)
			{
                string strImage = string.Format("BackImage/bg_ink0{0}.jpg", i);
                Uri uri = new Uri(strImage, UriKind.Relative);

                _listImageUri.Add(uri);
			}

            _MaxBackImage = _listImageUri.Count;
        }

        Uri GetImageRandom()
        {
            // 0과 _MaxBackImage
            int nIndex = _oRandom.Next(0, _MaxBackImage);

            return _listImageUri[nIndex];
        }

        void CreateInkEditor()
        {
            // 새 InkPresenter Editor 생성
            _oInkEditor = new UCInk(_szInk, true);
            _oInkEditor.BackImage = GetImageRandom();

            // Canvas에 추가
            cvEditor.Children.Add(_oInkEditor);
        }

        void ConnectEditorEvents()
        {
            // Ink Presenter 동작 이벤트
            cvUndo.MouseLeftButtonUp += new MouseEventHandler(cvUndo_MouseLeftButtonUp);
            cvRedo.MouseLeftButtonUp += new MouseEventHandler(cvRedo_MouseLeftButtonUp);
            cvClear.MouseLeftButtonUp += new MouseEventHandler(cvClear_MouseLeftButtonUp);
            cvChangeBG.MouseLeftButtonUp += new MouseEventHandler(cvChangeBG_MouseLeftButtonUp);
            cvSave.MouseLeftButtonUp += new MouseEventHandler(cvSave_MouseLeftButtonUp);
        }

        void cvUndo_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            _oInkEditor.Undo();
        }

        void cvRedo_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            _oInkEditor.Redo();
        }

        void cvClear_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            _oInkEditor.Clear(false);
        }

        void cvChangeBG_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            _oInkEditor.BackImage = GetImageRandom();
        }

        void cvSave_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            // cvEditor에서 제거 : 은퇴
            cvEditor.Children.Clear();

            // 현재 InkPresenter Editor를 편집 불가로 만들어서 InkWork에 추가
            _oInkEditor.Editable = false;
            _listInkWork.Add(_oInkEditor);

            // 새 편집용 UCInk 생성
            CreateInkEditor();

            // InkList 업데이트
            UpdateInkList();
        }

        void UpdateInkList()
        {
            cvInkList.Children.Clear();

            int nX = 1;
            int nY = 1;

            int nMax = 3; // 한줄에 3개 배치
            Size szMargin = new Size(10, 10); // 작품 간의 Margin

            int nCount = _listInkWork.Count;

            for (int i = 0; i < nCount; i++)
            {
                int nIndex = nCount - i - 1;
                UCInk oUCInk = _listInkWork[nIndex];

                // cvInkList 캔버스에 추가
                cvInkList.Children.Add(oUCInk);

                // 위치 조정
                oUCInk.SetValue(Canvas.LeftProperty, (_szInk.Width + szMargin.Width) * (nX - 1));
                oUCInk.SetValue(Canvas.TopProperty, (_szInk.Height + szMargin.Height) * (nY - 1));

                // 연배 증가
                nX++;

                if (nX > nMax) // 한 줄이 포화상태에 도달하면
                {
                    nX = 1;
                    nY++; // 다음 줄로
                }
            }
        }
    }
}
