﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace InkPresentationSample_UserControl
{
    public class UCInk : Control
    {
        InkPresenter _ink;
        Canvas _cvMyself;

        public bool Editable { get; set; }

        Uri _BackImage = null;
        public Uri BackImage { get { return _BackImage; } set { SetBackgroundImage(value); } }

        // Ink 쓰기와 관련
        Stroke _strokeCurrent = null;
        StrokeCollection _oUndoBuffer = new StrokeCollection();

        // 기본 생성자
        public UCInk(Size sz, bool bEditable)
        {
            System.IO.Stream s = this.GetType().Assembly.GetManifestResourceStream("InkPresentationSample_UserControl.UCInk.xaml");
            _cvMyself = this.InitializeFromXaml(new System.IO.StreamReader(s).ReadToEnd()) as Canvas;

            _ink = _cvMyself.FindName("ink") as InkPresenter;

            // 사이즈 설정
            UpdateSize(sz);

            // 편집 가능 여부
            Editable = bEditable;

            // InkPresenter Mouse Event 필수 4종세트
            _ink.MouseLeftButtonDown += new MouseEventHandler(ink_MouseLeftButtonDown);
            _ink.MouseMove += new MouseEventHandler(ink_MouseMove);
            _ink.MouseLeftButtonUp += new MouseEventHandler(ink_MouseLeftButtonUp);
            _ink.MouseLeave += new EventHandler(ink_MouseLeave);
        }

        void ink_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            if (Editable == false)
                return;

            _ink.CaptureMouse();
            StartNewStroke();
        }

        void ink_MouseMove(object sender, MouseEventArgs e)
        {
            if (_strokeCurrent == null)
                return;

            _strokeCurrent.StylusPoints.AddStylusPoints(e.GetStylusPoints(_ink));
        }

        void ink_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            _ink.ReleaseMouseCapture();
            _strokeCurrent = null;
        }

        void ink_MouseLeave(object sender, EventArgs e)
        {
            _strokeCurrent = null;
        }

        Stroke AllocStroke()
        {
            // 스트로크 준비 흰색
            Stroke stroke = new Stroke();
            stroke.DrawingAttributes.Color = Colors.White;
            stroke.DrawingAttributes.OutlineColor = Colors.Black;
            stroke.DrawingAttributes.Width = 2;
            stroke.DrawingAttributes.Height = 2;

            return stroke;
        }

        void StartNewStroke()
        {
            _strokeCurrent = AllocStroke();
            _ink.Strokes.Add(_strokeCurrent);

            // Undo, Redo 버퍼 초기화
            Flush_oUndoBuffer();
        }

        void Flush_oUndoBuffer()
        {
            _oUndoBuffer.Clear();
        }

        void SetBackgroundImage(Uri uri)
        {
            if (uri == null)
                return;

            _BackImage = uri;

            ImageBrush ib = new ImageBrush();
            ib.ImageSource = _BackImage;
            _ink.Background = ib;
        }

        void UpdateSize(Size sz)
        {
            // Control 사이즈
            Width = sz.Width;
            Height = sz.Height;

            // 무시해도 좋지만 해주면 좋음
            _cvMyself.Width = Width;
            _cvMyself.Height = Height;

            // InkPresenter 사이즈
            _ink.Width = Width;
            _ink.Height = Height;
        }

        public void Undo()
        {
            if (_ink.Strokes.Count == 0)
                return;

            Stroke oUndo = _ink.Strokes[_ink.Strokes.Count - 1];
            _oUndoBuffer.Add(oUndo);
            _ink.Strokes.RemoveAt(_ink.Strokes.Count - 1);
        }

        public void Redo()
        {
            if (_oUndoBuffer.Count == 0)
                return;

            Stroke oRedo = _oUndoBuffer[_oUndoBuffer.Count - 1];
            _ink.Strokes.Add(oRedo);
            _oUndoBuffer.RemoveAt(_oUndoBuffer.Count - 1);
        }

        public void Clear(bool bClearBackImage)
        {
            _ink.Strokes.Clear();
            if (bClearBackImage == true)
            {
                _ink.Background = null;
            }

            Flush_oUndoBuffer();
        }
    }
}
