﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Interop; // BrowserHost 사용을 위해
using System.Windows.Browser; // HtmlPage 사용을 위해 [링크]

namespace InkPresenterSample
{
    public partial class Page : Canvas
    {
        // 스트로크
        Stroke _strokeCurrent = null;

        public void Page_Loaded(object o, EventArgs e)
        {
            // Required to initialize variables
            InitializeComponent();

            // 화면리사이즈 이벤트
            BrowserHost.Resize += new EventHandler(BrowserHost_Resize);

            // ink에 이미지 브러쉬로 Fill
            ImageBrush ib = new ImageBrush();
            ib.ImageSource = new Uri("BG_InkPresenter.jpg", UriKind.Relative);
            ink.Background = ib;

            // 각종 링크 달기
            AddLink();

            // InkPresenter Mouse Event 필수 4종세트
            ink.MouseLeftButtonDown += new MouseEventHandler(ink_MouseLeftButtonDown);
            ink.MouseMove += new MouseEventHandler(ink_MouseMove);
            ink.MouseLeftButtonUp += new MouseEventHandler(ink_MouseLeftButtonUp);
            ink.MouseLeave += new EventHandler(ink_MouseLeave);
        }

        void BrowserHost_Resize(object sender, EventArgs e)
        {
            // Root Canvas가 화면을 가득채우게 함.
            Width = BrowserHost.ActualWidth;
            Height = BrowserHost.ActualHeight;

            // Content Canvas가 정중앙으로 오도록 함.
            AlignCenter(cvContent);
        }

        void AddLink()
        {
            // Gilbert Blog
            tbGilbert.MouseLeftButtonUp += delegate
            {
                HtmlPage.Navigate("http://gilverlight.tistory.com", "_blank"); // 새 창으로 열기
            };

            // 실버라이트 카페 (네이버)
            cvSLCafe.MouseLeftButtonUp += delegate
            {
                HtmlPage.Navigate("http://cafe.naver.com/mssilverlight", "_blank"); // 새 창으로 열기
            };

            // 우리 회사 휴즈플로우
            imgHugeFlow.MouseLeftButtonUp += delegate
            {
                HtmlPage.Navigate("http://hugeflow.com", "_blank"); // 새 창으로 열기
            };
        }

        void AlignCenter(Canvas cv)
        {
            cv.SetValue(Canvas.LeftProperty, (Width - cv.Width) / 2);
            cv.SetValue(Canvas.TopProperty, (Height - cv.Height) / 2);
        }

        void ink_MouseLeave(object sender, EventArgs e)
        {
            _strokeCurrent = null;
        }

        void ink_MouseLeftButtonDown(object sender, MouseEventArgs e)
        {
            ink.CaptureMouse();
            StartNewStroke();
        }

        void ink_MouseMove(object sender, MouseEventArgs e)
        {
            if (_strokeCurrent != null)
            {
                _strokeCurrent.StylusPoints.AddStylusPoints(e.GetStylusPoints(ink));
            }
        }

        void ink_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            ink.ReleaseMouseCapture();
            _strokeCurrent = null;
        }

        Stroke AllocStroke()
        {
            // 스트로크 준비 흰색
            Stroke stroke = new Stroke();
            stroke.DrawingAttributes.Color = Colors.White;
            stroke.DrawingAttributes.OutlineColor = Colors.Black;
            stroke.DrawingAttributes.Width = 3;
            stroke.DrawingAttributes.Height = 3;

            return stroke;
        }

        void StartNewStroke()
        {
            _strokeCurrent = AllocStroke();
            ink.Strokes.Add(_strokeCurrent);
        }
    }
}
